@@ -30,12 +30,23 @@ test_that("globals work", {
     expect_error(chkFle("no_file"), regexp = "^File \".*?\" not found.")
     expect_error(chkFle(nmeOMV, fleCnt = "no_file"), regexp = "^chkFle: File \".*?\" doesn't contain the file \".*?\"\\.")
     expect_error(chkVar(dtaFrm = data.frame(A = runif(100)), varNme = c("A", "B")), regexp = "^The variable\\(s\\) \\w+ are not contained in the current data set\\.")
+
+    set.seed(1)
+    tmpDF <- setAtt("value.labels", list(value.labels = setNames(seq(4), sprintf("Level %d", seq(4)))), data.frame(tmpCol = sample(seq(4), 200, replace = TRUE)))
+    tmpDF[, 1] <- cnvCol(tmpDF[, 1], "factor")
+    expect_s3_class(tmpDF[, 1], "factor")
+    expect_equal(levels(tmpDF[, 1]), sprintf("Level %d", seq(4)))
+    expect_equal(as.integer(table(tmpDF[, 1])), c(51, 54, 51, 44))
+
     expect_error(fmtFlI(fleInp = tempfile(), minLng = 2), regexp = "^The fleInp-argument is supposed to be a character vector with a minimal length of \\d+ and a maximal length of")
     expect_error(fmtFlI(fleInp = c(tempfile(), tempfile()), maxLng = 1),
       regexp = "^The fleInp-argument is supposed to be a character vector with a minimal length of \\d+ and a maximal length of")
-    expect_error(fmtFlO(fleOut = ""),          regexp = "^fleOut needs to be a valid non-empty file name \\(character\\), and the file extension for output file needs to be \\.omv\\.")
-    expect_error(fmtFlO(fleOut = "Trial"),     regexp = "^fleOut needs to be a valid non-empty file name \\(character\\), and the file extension for output file needs to be \\.omv\\.")
-    expect_error(fmtFlO(fleOut = "Trial.rds"), regexp = "^fleOut needs to be a valid non-empty file name \\(character\\), and the file extension for output file needs to be \\.omv\\.")
+    expect_error(fmtFlO(fleOut = ""),
+      regexp = "^fleOut needs to be a valid non-empty file name \\(character\\), and the file extension for output file needs to be \\.omv\\ or \\.omt\\.")
+    expect_error(fmtFlO(fleOut = "Trial"),
+      regexp = "^fleOut needs to be a valid non-empty file name \\(character\\), and the file extension for output file needs to be \\.omv\\ or \\.omt\\.")
+    expect_error(fmtFlO(fleOut = "Trial.rds"),
+      regexp = "^fleOut needs to be a valid non-empty file name \\(character\\), and the file extension for output file needs to be \\.omv\\ or \\.omt\\.")
     expect_error(fcnArg(c("stats::sd", "stats::mean", "C")), regexp = "^The argument to fcnArg must be a character \\(vector\\) with 1 or 2 elements.")
     expect_true(jmvPtB())
     tmpPB <- var2PB(inpVar = list(list(A = NULL, B = TRUE, C = 1, D = 0.01, E = "Trial", F = c(TRUE, FALSE, TRUE), G = c(1, 2, 3), H = c(0.01, 0.02, 0.03), I = c("A", "B"))))
@@ -65,6 +76,13 @@ test_that("globals work", {
     expect_equal(setdiff(names(attributes(jmvReadWrite::AlbumSales)), names(attributes(rmvAtt(jmvReadWrite::AlbumSales)))), c("datalabel", "var.labels"))
     unlink(nmeOMV)
 
+    tmpDF <- rstAtt(jmvReadWrite::AlbumSales, c("datalabel", "var.labels"))
+    expect_equal(names(attributes(tmpDF)), names(attributes(jmvReadWrite::AlbumSales)))
+    expect_equal(attr(tmpDF, "datalabel"),  character(0))
+    expect_equal(attr(tmpDF, "var.labels"), character(0))
+    expect_equal(length(attr(tmpDF, "datalabel")),  0)
+    expect_equal(length(attr(tmpDF, "var.labels")), 0)
+
     expect_error(inp2DF(dtaInp = 1), regexp = "^dtaInp must either be a data frame or a character \\(pointing to a location where the input file can be found\\)\\.")
     inpDF <- jmvReadWrite::AlbumSales
     inpNme <- file.path("..", "ToothGrowth.omv")
@@ -120,21 +138,24 @@ test_that("globals work", {
     expect_equal(names(df4Chk), row.names(df4Chk))
     expect_equal(names(df4Chk), sprintf("V%d", seq(10)))
     expect_false(any(is.na(df4Chk)))
-    expect_equal(unname(colMeans(df4Chk)), c(0.128183893, 0.096625131, 0.070568445, 0.115919003, 0.139886614, 0.058323693, 0.092441925, 0.106711615, 0.168887776, 0.10403618))
+    expect_equal(unname(colMeans(df4Chk)),
+      c(0.128183893, 0.096625131, 0.070568445, 0.115919003, 0.139886614, 0.058323693, 0.092441925, 0.106711615, 0.168887776, 0.10403618))
 
     df4Chk <- mtxF2S(tmpDF, rmvTrU = TRUE)
     expect_equal(dim(df4Chk), c(10, 10))
     expect_equal(names(df4Chk), row.names(df4Chk))
     expect_equal(names(df4Chk), sprintf("V%d", seq(10)))
     expect_identical(as.integer(colSums(is.na(df4Chk))), seq(0, 9))
-    expect_equal(unname(colMeans(df4Chk, na.rm = TRUE)), c(0.12818389, 0.10747174, 0.09210481, 0.16532417, 0.17717275, 0.18858648, 0.25467175, 0.34689183, 0.59969889, 1))
+    expect_equal(unname(colMeans(df4Chk, na.rm = TRUE)),
+      c(0.12818389, 0.10747174, 0.09210481, 0.16532417, 0.17717275, 0.18858648, 0.25467175, 0.34689183, 0.59969889, 1))
 
     df4Chk <- mtxF2S(tmpDF, rmvDgn = TRUE)
     expect_equal(dim(df4Chk), c(10, 10))
     expect_equal(names(df4Chk), row.names(df4Chk))
     expect_equal(names(df4Chk), sprintf("V%d", seq(10)))
     expect_identical(unname(colSums(is.na(df4Chk))), rep(1, 10))
-    expect_equal(unname(colMeans(df4Chk, na.rm = TRUE)), c(0.031315436, -0.003749854, -0.032701727, 0.017687781, 0.044318459, -0.046307008, -0.008397862, 0.007457349, 0.076541973, 0.004484645))
+    expect_equal(unname(colMeans(df4Chk, na.rm = TRUE)),
+      c(0.031315436, -0.003749854, -0.032701727, 0.017687781, 0.044318459, -0.046307008, -0.008397862, 0.007457349, 0.076541973, 0.004484645))
 
     df4Chk <- mtxF2S(tmpDF, rmvTrU = TRUE, mtxXps = TRUE)
     expect_equal(dim(df4Chk), c(10, 10))
@@ -148,12 +169,14 @@ test_that("globals work", {
     expect_equal(names(df4Chk), row.names(df4Chk))
     expect_equal(names(df4Chk), sprintf("V%d", seq(10)))
     expect_identical(as.integer(colSums(is.na(df4Chk))), seq(10, 1))
-    expect_equal(unname(colMeans(df4Chk, na.rm = TRUE)), c(NA, -0.0009943199, -0.0155770134, 0.0006402703, 0.0839574127, -0.0719390890, -0.0157112896, 0.0037772361, 0.0611849980, 0.0044846446))
+    expect_equal(unname(colMeans(df4Chk, na.rm = TRUE)),
+      c(NA, -0.0009943199, -0.0155770134, 0.0006402703, 0.0839574127, -0.0719390890, -0.0157112896, 0.0037772361, 0.0611849980, 0.0044846446))
 
     df4Chk <- mtxF2S(tmpDF, mtxSps = TRUE)
     expect_equal(dim(df4Chk), c(9, 10))
     expect_equal(names(df4Chk), c("Variable", sprintf("V%d", seq(1, 9))))
     expect_equal(row.names(df4Chk), sprintf("V%d", seq(2, 10)))
     expect_identical(unname(colSums(is.na(df4Chk))), c(0, seq(0, 8)))
-    expect_equal(unname(colMeans(df4Chk[, -1], na.rm = TRUE)), c(0.031315436, -0.004094296, -0.037594503,  0.026211536, 0.012607297, -0.014266906, 0.006228995, 0.020337746, 0.199397772))
+    expect_equal(unname(colMeans(df4Chk[, -1], na.rm = TRUE)),
+      c(0.031315436, -0.004094296, -0.037594503,  0.026211536, 0.012607297, -0.014266906, 0.006228995, 0.020337746, 0.199397772))
 })
