@@ -100,15 +100,15 @@ test_that("merge_cols_omv works", {
     expect_true(chkFle(nmeOut, fleCnt = "data.bin"))
     df4Chk <- read_omv(nmeOut, sveAtt = FALSE, getSyn = TRUE)
     expect_s3_class(df4Chk, "data.frame")
-    expect_equal(dim(df4Chk), c(60, 19))
+    expect_equal(dim(df4Chk), c(60, 18))
     expect_equal(names(df4Chk),
-      c("ID", "Filter 1", "logLen", "supp - Transform 1", "len", "supp", "dose", "dose2", "dose3", "Trial", "Residuals", "J", "K", "L", "M", "O", "weights", "A", "B"))
+      c("ID", "Filter 1", "logLen", "supp - Transform 1", "len", "supp", "dose", "dose2", "dose3", "Trial", "Residuals", "J", "K", "L", "M", "weights", "A", "B"))
     expect_equal(vapply(df4Chk, typeof, character(1), USE.NAMES = FALSE),
-      c("integer", "logical", "double", "integer", "double", "integer", "double", "integer", "integer", "integer",
-        "double", "double", "double", "integer", "logical", "logical", "integer", "double", "double"))
+      c("character", "logical", "double", "integer", "double", "integer", "double", "integer", "integer", "integer",
+        "double", "double", "double", "integer", "logical", "integer", "double", "double"))
     expect_equal(sort(zip::zip_list(nmeOut)$filename),
       c("01 empty/analysis", "02 anova/analysis", "02 anova/resources/65167cb3bdaf8761.png", "02 anova/resources/99f9b5d34a92049b.png", "03 empty/analysis",
-        "04 ancova/analysis", "05 empty/analysis", "data.bin", "index.html", "meta", "metadata.json", "xdata.json"))
+        "04 ancova/analysis", "05 empty/analysis", "data.bin", "index.html", "meta", "metadata.json", "strings.bin", "xdata.json"))
     expect_equal(attr(df4Chk, "syntax"),
       c(paste("jmv::ANOVA(formula = len ~ supp + dose2 + supp:dose2, data = data, effectSize = \"partEta\", modelTest = TRUE, qq = TRUE,",
               "contrasts = list(list(var=\"supp\", type=\"none\"), list(var=\"dose2\", type=\"polynomial\")), postHoc = ~ supp + dose2, emMeans = ~ dose2:supp)"),
