@@ -1,7 +1,7 @@
 test_that("write_omv works", {
     # check whether writing the data is working (file existence, size, contents [.omv-files are ZIP archives and must contain files that include meta, metadata.json, data.bin])
     dtaOut <- jmvReadWrite::ToothGrowth
-    colOut <- dim(dtaOut)[1]
+    rowOut <- dim(dtaOut)[1]
     nmeOut <- tempfile(fileext = ".omv")
     dtaDbg <- write_omv(dtaFrm = dtaOut, fleOut = nmeOut, retDbg = TRUE)
     expect_true(file.exists(nmeOut))
@@ -60,13 +60,73 @@ test_that("write_omv works", {
     expect_equal(c(lapply(jmvReadWrite::ToothGrowth, class), list(T1 = "integer", T2 = "numeric", T3 = "factor", T4 = c("ordered", "factor"), T5 = "factor", T6 = "factor")),
                  lapply(dtaDbg$dtaFrm, class))
 
+    nmeTmp <- tempfile(fileext = ".omt")
+    write_omv(dtaFrm = dtaOut, fleOut = nmeTmp)
+    expect_true(file.exists(nmeTmp))
+    expect_gt(file.info(nmeTmp)$size, 1)
+    expect_true(chkFle(nmeTmp, isZIP = TRUE))
+    expect_true(chkFle(nmeTmp, fleCnt = "meta"))
+    expect_true(chkFle(nmeTmp, fleCnt = "metadata.json"))
+    expect_true(chkFle(nmeTmp, fleCnt = "data.bin"))
+    dtaTmp <- read_omv(nmeTmp)
+    expect_s3_class(dtaTmp, "data.frame")
+    expect_equal(dim(dtaTmp), c(0, 7))
+    expect_equal(names(attributes(dtaTmp)), c("names", "row.names", "removedRows", "addedRows", "transforms", "class"))
+    expect_equal(names(attributes(dtaTmp[[3]]))[1:10],
+      c("levels", "class", "values", "jmv-desc", "name", "id", "columnType", "dataType", "measureType", "formula"))
+    expect_equal(names(attributes(dtaTmp[[7]]))[1:10],
+      c("jmv-desc", "name", "id", "columnType", "dataType", "measureType", "formula", "formulaMessage", "parentId", "width"))
+    expect_equal(unname(unlist(attributes(dtaTmp[[1]]))),
+      c("TRUE", "ID", "1", "Data", "Text", "ID", "", "", "0", "100", "string", "", "", "0"))
+    expect_equal(unname(unlist(attributes(dtaTmp[[4]]))),
+      c("dose", "4", "Data", "Decimal", "Continuous", "", "", "0", "100", "number", "", "", "0"))
+    expect_equal(lapply(jmvReadWrite::ToothGrowth, class), lapply(dtaTmp, class))
+    unlink(nmeTmp)
+
+    nmeTmp <- tempfile(fileext = ".omt")
+    # NB: edits (in addCol) and addedRows should be reset (empty object of the same type, i.e., list())
+    addCol <- setAtt(c("columnType", "dataType", "measureType", "formula", "edits"),
+                     list(columnType = "Computed", dataType = "Decimal", measureType = "Continuous", formula = "LOG10(len)", edits = list(start = 0, end = 499)),
+                     setNames(dtaOut[, 7, FALSE], "logLenC"))
+    write_omv(dtaFrm = setAtt("addedRows", list(addedRows = list(start = 0, end = 499)), cbind(dtaOut, addCol)), fleOut = nmeTmp)
+    expect_true(file.exists(nmeTmp))
+    expect_gt(file.info(nmeTmp)$size, 1)
+    expect_true(chkFle(nmeTmp, isZIP = TRUE))
+    expect_true(chkFle(nmeTmp, fleCnt = "meta"))
+    expect_true(chkFle(nmeTmp, fleCnt = "metadata.json"))
+    expect_true(chkFle(nmeTmp, fleCnt = "data.bin"))
+    dtaTmp <- read_omv(nmeTmp)
+    expect_s3_class(dtaTmp, "data.frame")
+    expect_equal(dim(dtaTmp), c(0, 8))
+    expect_equal(attributes(dtaTmp), list(names = c("ID", "supp", "supp2", "dose", "dose2", "len", "logLen", "logLenC"), row.names = integer(0),
+                                          removedRows = list(), addedRows = list(), transforms = list(), class = "data.frame"))
+    expect_equal(names(attributes(dtaTmp[[3]]))[1:10],
+      c("levels", "class", "values", "jmv-desc", "name", "id", "columnType", "dataType", "measureType", "formula"))
+    expect_equal(names(attributes(dtaTmp[[7]]))[1:10],
+      c("jmv-desc", "name", "id", "columnType", "dataType", "measureType", "formula", "formulaMessage", "parentId", "width"))
+    expect_equal(names(attributes(dtaTmp[[8]]))[1:10],
+      c("jmv-desc", "name", "id", "columnType", "dataType", "measureType", "formula", "formulaMessage", "parentId", "width"))
+    expect_equal(unname(unlist(attributes(dtaTmp[[1]]))),
+      c("TRUE", "ID", "1", "Data", "Text", "ID", "", "", "0", "100", "string", "", "", "0"))
+    expect_equal(unname(unlist(attributes(dtaTmp[[4]]))),
+      c("dose", "4", "Data", "Decimal", "Continuous", "", "", "0", "100", "number", "", "", "0"))
+    expect_equal(unname(unlist(attributes(dtaTmp[[7]]))),
+      c("Natural logarithm of the tooth length (len)", "logLen", "7", "Data", "Decimal", "Continuous",
+        "",           "", "0", "100", "number", "", "Natural logarithm of the tooth length (len)", "0"))
+    expect_equal(unname(unlist(attributes(dtaTmp[[8]]))),
+      c("Natural logarithm of the tooth length (len)", "logLenC", "8", "Computed", "Decimal", "Continuous",
+        "LOG10(len)", "", "0", "100", "number", "", "Natural logarithm of the tooth length (len)", "0"))
+    expect_equal(c(lapply(jmvReadWrite::ToothGrowth, class), list(logLenC = "numeric")), lapply(dtaTmp, class))
+    unlink(nmeTmp)
+
     # test cases for code coverage ============================================================================================================================
     expect_error(write_omv(NULL, nmeOut), regexp = "^The data frame to be written needs to be given as parameter \\(dtaFrm = \\.\\.\\.\\)\\.")
     expect_error(write_omv(data.frame(T1 = sample(9999, 100), T2 = as.complex(rnorm(100))), nmeOut),
       regexp = "Variable type \\w+ not implemented\\. Please send the data file that caused this problem to sebastian\\.jentschke@uib\\.no")
     expect_error(write_omv(dtaDbg$dtaFrm), regexp = "^Output file name needs to be given as parameter \\(fleOut = \\.\\.\\.\\)\\.")
     expect_error(add2ZIP(fleZIP = nmeOut, crrHdl = NULL),
-      regexp = "fleZIP \\(a character with a file name\\), and either crrHdl \\(with a connection\\) or crrFle \\(with a file name and \\[optionally\\] a writing mode\\) need to be given as arguments.")
+      regexp = paste0("fleZIP \\(a character with a file name\\), and either crrHdl \\(with a connection\\) or crrFle \\(with a file name",
+                      " and \\[optionally\\] a writing mode\\) need to be given as arguments."))
     expect_error(add2ZIP(fleZIP = nmeOut, crrHdl = 1),
       regexp = "^Parameter isn't a file handle pointing to a file to be zipped\\:")
 
@@ -98,17 +158,23 @@ test_that("write_omv works", {
     unlink(nmeOut)
 
     set.seed(1)
-    dtaOut <- cbind(dtaOut, data.frame(Bool = sample(c(TRUE, FALSE), colOut, TRUE),
-                                       Date = sample(seq(as.Date("1999/01/01"), as.Date("2000/01/01"), by = "day"), colOut),
-                                       Time = sample(as.difftime(tim = seq(0, 3600), units = "secs"), colOut)))
+    dtaOut <- cbind(dtaOut, data.frame(Bool  =            sample(c(TRUE, FALSE), rowOut, TRUE),
+                                       Date1 =            sample(seq(as.Date("1999/01/01"), as.Date("2000/01/01"), by = "day"), rowOut),
+                                       Date2 = as.POSIXct(sample(seq(as.Date("1999/01/01"), as.Date("2000/01/01"), by = "day"), rowOut)),
+                                       Date3 = as.POSIXlt(sample(seq(as.Date("1999/01/01"), as.Date("2000/01/01"), by = "day"), rowOut)),
+                                       Time  =            sample(as.difftime(tim = seq(0, 3600), units = "secs"), rowOut)))
     write_omv(dtaFrm = dtaOut, fleOut = nmeOut)
     dtaInp <- read_omv(fleInp = nmeOut)
     unlink(nmeOut)
     expect_equal(as.integer(table(dtaInp[["Bool"]])), c(29, 31))
-    expect_equal(c(mean(dtaInp[["Date"]]), sd(dtaInp[["Date"]])), c(10787.3667, 108.7002), tolerance = 1e-4)
-    expect_equal(attr(dtaInp[["Date"]], "jmv-desc"), "Date (date converted to numeric; days since 1970-01-01)")
-    expect_equal(c(mean(dtaInp[["Time"]]), sd(dtaInp[["Time"]])), c(1538.367, 1041.579), tolerance = 1e-4)
-    expect_equal(attr(dtaInp[["Time"]], "jmv-desc"), "Time (time converted to numeric; sec since 00:00)")
+    expect_equal(c(mean(dtaInp[["Date1"]]), sd(dtaInp[["Date1"]])), c(10787.3667, 108.7002), tolerance = 1e-4)
+    expect_equal(attr(dtaInp[["Date1"]], "jmv-desc"), "Date1 (date converted to integer; days since 1970-01-01)")
+    expect_equal(c(mean(dtaInp[["Date2"]]), sd(dtaInp[["Date2"]])), c(10767.2330, 108.2620), tolerance = 1e-4)
+    expect_equal(attr(dtaInp[["Date2"]], "jmv-desc"), "Date2 (date converted to integer; days since 1970-01-01)")
+    expect_equal(c(mean(dtaInp[["Date3"]]), sd(dtaInp[["Date3"]])), c(10762.8000, 112.2584), tolerance = 1e-4)
+    expect_equal(attr(dtaInp[["Date3"]], "jmv-desc"), "Date3 (date converted to integer; days since 1970-01-01)")
+    expect_equal(c(mean(dtaInp[["Time"]]),  sd(dtaInp[["Time"]])),   c(2090.8000, 978.9000), tolerance = 1e-4)
+    expect_equal(attr(dtaInp[["Time"]], "jmv-desc"), "Time (time converted to integer; sec since 00:00)")
     dtaOut <- read_omv(file.path("..", "ToothGrowth.omv"))
     attr(dtaOut, "jmv-weights-name") <- "weights"
     attr(dtaOut, "jmv-weights") <- as.vector(dtaOut[, "weights"])
@@ -185,9 +251,17 @@ test_that("write_omv works", {
     expect_equal(unlist(attributes(jmvAtt(tmpDF)[["NN"]]), use.names = FALSE), c(sprintf("%d", seq(1:7)), "factor", sprintf("%d", seq(1:7)), "Nominal", "Integer"))
     expect_equal(unlist(attributes(jmvAtt(tmpDF)[["CR"]]), use.names = FALSE), c("Trial (is description kept?)", "Nominal", "Text"))
     expect_error(jmvAtt("Trial"),      regexp = "^Input data are either not a data frame or have incorrect \\(only one or more than two\\) dimensions\\.")
-    expect_error(jmvAtt(data.frame()), regexp = "^The first dimension of the input data frame has not the required size \\(0 < 1\\)\\.")
+    expect_error(jmvAtt(data.frame()), regexp = "^The second dimension of the input data frame has not the required size \\(0 < 1\\)\\.")
     expect_error(jmvAtt(cbind(tmpDF, data.frame(EC = sample(as.complex(seq(10)), 100, replace = TRUE)))), regexp = "^\\s+\\w+: Variable type \\w+ not implemented.")
 
+    tmpDF  <- structure(list(value = structure(c(1, 2, 4), format.sas = "LEVELS", class = c("haven_labelled", "vctrs_vctr", "double"),
+                             labels = c(level1 = 1, level2 = 2, level3 = 4))), class = c("tbl_df", "tbl", "data.frame"), row.names = c(NA, -3L))
+    df4Chk <- write_omv(dtaFrm = tmpDF, fleOut = nmeOut, retDbg = TRUE)$dtaFrm
+    unlink(nmeOut)
+    expect_equal(attributes(df4Chk), list(names = "value", row.names = seq(3), class = "data.frame"))
+    expect_equal(attributes(df4Chk[, "value"]),
+        list(levels = c("level1", "level2", "level3"), class = "factor", measureType = "Nominal", dataType = "Text"))
+
     tmpCol <- rnorm(100)
     attr(tmpCol, "Trial") <- "Check whether attribute is preserved"
     expect_equal(cnvCol(tmpCol, tgtTyp = "integer"), round(tmpCol))
